#!/usr/bin/python3
# -*-coding:Utf-8 -*
#  version 3.2.1
#  calcul de la fréquence plus précis par un mix FFT et autocorrélation
#  page filtrage ajoutée
#  fonctionnement des curseurs amélioré
#  corrigé la confusion entre le module scipy.fft et la fonction fft
#  compatible avec TBS 1072B-EDU 
#  Déplacement du 'try' dans trouvevisa
#  on peut entrer des valeurs décimales pour la fréquence d'affinage et correction d'un bug en cas de fréquence incorrecte
#  les labels des axes de droite ont été enlevés car ils s'affichaient à gauche (non résolu)

import sys
import csv
import pyvisa as visa
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from PyQt5.QtWebEngineWidgets import QWebEngineView
from PyQt5.QtGui import QScreen, QGuiApplication
from numpy import *
from scipy.signal import find_peaks
#  gestion du changement de module scipy
try:
    from scipy.fft import fft as scfft
except ModuleNotFoundError:
    from scipy import fft as scfft
import PyQt5.QtGui
import matplotlib.pyplot as plt
import os


class aide(QWidget):
    def __init__(self, url, parent=None):
        super(aide, self).__init__(parent)

        self.setWindowTitle("Aide")

        browser = QWebEngineView()
        ad = "file:///" + os.getcwd().replace("\\", "/") + url  #  créer le chemin absolu
        browser.load(QUrl(ad))

        layout = QVBoxLayout()
        layout.addWidget(browser)

        self.setLayout(layout)
        self.show()

class operations(QWidget):
    def __init__(self, **kwargs):
        #  super(operations, self).__init__(parent)
        QWidget.__init__(self)

        self.setWindowTitle("Opérations entre les voies")

        #  zone de définition de l'opération
        f = 50
        if main.f1 != 0.0: f = main.f1
        frame_formule = QFrame()
        frame_formule.setFrameStyle(34)
        titre = QLabel('Opération entre les voies ou fonction du temps :')
        self.formule = QLineEdit('CH1-CH2 + 10*sin(2*pi*' + str(f) + '*t)', frame_formule)
        self.Bexecuter = QPushButton('Exécuter', frame_formule)
        self.Bexecuter.clicked.connect(self.calculer)
        self.Baide = QPushButton('Aide', frame_formule)
        self.Baide.clicked.connect(self.lance_aide)

        # a figure instance to plot on
        self.figureO = plt.figure()
        self.figureO.patch.set_facecolor('white')

        # this is the Canvas Widget that displays the `figure`
        # it takes the `figure` instance as a parameter to __init__
        self.canvasO = FigureCanvas(self.figureO)
        # create an axis
        self.axCH1 = self.figureO.add_subplot(211)
        self.axCH2 = self.axCH1.twinx()

        # create an other axis
        self.axop = self.figureO.add_subplot(212)

        # this is the Navigation widget
        # it takes the Canvas widget and a parent
        self.toolbarO = NavigationToolbar(self.canvasO, self)

        #  bouton copier
        bouton_copier = QPushButton('Copier')
        bouton_copier.clicked.connect(self.copier)

        #  remplacer une voie par le résultat
        frame_remplacer = QFrame()
        titre_r = QLabel('Remplacer, dans la page principale, une voie par le résultat :')
        self.RB_CH1 = QRadioButton('CH1', frame_remplacer)
        self.RB_CH2 = QRadioButton('CH2', frame_remplacer)
        self.RB_aucune = QRadioButton('Aucune (restitution des voies)', frame_remplacer)
        self.RB_CH1.toggled.connect(self.remplacer)
        self.RB_CH2.toggled.connect(self.remplacer)

        layout = QVBoxLayout()

        layout_groupe = QVBoxLayout()
        layout_groupe.addWidget(titre)
        layoutFormule = QHBoxLayout()
        layoutFormule.addWidget(self.formule, stretch=1)
        layoutFormule.addWidget(self.Bexecuter)
        layoutFormule.addWidget(self.Baide)
        layout_groupe.addLayout(layoutFormule)
        frame_formule.setLayout(layout_groupe)

        layoutGraph = QVBoxLayout()  # contient le graphe et ses menus
        layoutGraph.addWidget(self.canvasO)
        layoutGraph.addWidget(self.toolbarO)

        layoutCopier = QHBoxLayout()
        layoutCopier.addStretch(1)
        layoutCopier.addWidget(bouton_copier)
        layoutCopier.addStretch(1)

        layout_remplacer = QHBoxLayout()
        layout_remplacer.addWidget(titre_r)
        layout_remplacer.addWidget(self.RB_CH1)
        layout_remplacer.addWidget(self.RB_CH2)
        layout_remplacer.addWidget(self.RB_aucune)
        frame_remplacer.setLayout(layout_remplacer)

        layout.addWidget(frame_formule)
        layout.addLayout(layoutGraph)
        layout.addLayout(layoutCopier)
        layout.addWidget(frame_remplacer)

        self.setLayout(layout)

        self.RB_aucune.setChecked(True)
        self.resultat = []
        self.titre = 'Résultat'
        self.calculer()
        self.show()

    def copier(self):
        screen = QGuiApplication.primaryScreen()
        p = screen.grabWindow(self.winId())
        pressPaper = QApplication.clipboard()
        pressPaper.setImage(p.toImage())

    def calculer(self):
        try:
            self.resultat= []
            for i in range(main.n):
                CH1 = main.voie1[i]
                CH2 = main.voie2[i]
                t = i * main.Te
                self.resultat.append(eval(self.formule.text()))

            self.titre = self.formule.text()
            self.plotO()
        except (NameError, SyntaxError) as e:
            self.message('Erreur de syntaxe')
        except ZeroDivisionError as e:
            self.message('division par 0 pendant les calculs')

    def lance_aide(self):
        main.lance_aide("/aide/aide-operations.html")

    def message(self, e):
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Critical)

        msg.setText("La formule n'est pas valide.")
        msg.setWindowTitle("Erreur")
        msg.setInformativeText(e)
        msg.setStandardButtons(QMessageBox.Ok)
        msg.setDefaultButton(QMessageBox.Ok)
        msg.exec()

    def remplacer(self):
        if self.RB_CH1.isChecked():
            main.voie1 = self.resultat
            #  main.voie2 = main.voie2_old

        if self.RB_CH2.isChecked():
            main.voie2 = self.resultat
            #  main.voie1 = main.voie1_old

        if self.RB_aucune.isChecked():
            main.voie2 = main.voie2_old
            main.voie1 = main.voie1_old

        main.mesures()
        main.plot()
        self.plotO()
        self.courbes_changees()

    def courbes_changees(self):
        #  rafraichit les autres pages si les signaux ont changé
        if main.spectres_ouvert: main.spectres()
        if main.filtrage_ouvert: main.fen_filtrage.calculer()

    def plotO(self):
        # discards the old graph
        self.axCH1.clear()
        self.axCH2.clear()
        self.axop.clear()
        Tt = arange(main.n) * main.Te
        nulle = linspace(0.0, 0.0, num=main.n)
        ntmax = main.n - 1

        # plot data

        self.axCH1.plot(Tt[:ntmax], nulle[:ntmax], 'Maroon', linewidth=1)
        self.axCH1.plot(Tt[:ntmax], main.voie1[:ntmax], 'red', linewidth=1, label='CH1')
        self.axCH2.plot(Tt[:ntmax], nulle[:ntmax], 'Navy', linewidth=1)
        self.axCH2.plot(Tt[:ntmax], main.voie2[:ntmax], 'blue', linewidth=1)
        #  self.axCH2.set_ylabel('CH2', color='blue')

        self.axop.plot(Tt[:ntmax], nulle[:ntmax], 'black', linewidth=1)
        self.axop.plot(Tt[:ntmax], self.resultat[:ntmax], 'green', linewidth=1, label='résultat')

        #self.axCH1.set_ylim([(-127 - main.Yoff1) * main.Ymul1, (127 - main.Yoff1) * main.Ymul1])
        #self.axCH2.set_ylim([(-127 - main.Yoff2) * main.Ymul2, (127 - main.Yoff2) * main.Ymul2])

        # légendes
        self.axCH1.set_ylabel("CH1 (V)", color='red')
        #  self.axCH2.set_ylabel("CH2 (V)")
        self.axCH1.set_xlabel("temps (s)")
        self.axop.set_ylabel('math', color='green')
        self.axop.set_xlabel("temps (s)")
        self.axCH1.grid(True)
        self.axop.grid(True)

        # refresh canvas
        self.canvasO.draw()

    def keyPressEvent(self, event):
        #  print('keypressEvent')
        #  réimplémente la méthode pour passer en manu en tapant entrée
        if type(event) == PyQt5.QtGui.QKeyEvent:
            if event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter:
                if self.formule.hasFocus():
                    self.calculer()

    def closeEvent(self, QCloseEvent):
        main.filtrage_ouvert = False


class filtrage(QWidget):
    def __init__(self, **kwargs):
        #  super(operations, self).__init__(parent)
        QWidget.__init__(self)

        self.setWindowTitle("Filtrage des voies")

        #  zone de définition du filtrage

        frame_choix_f = QFrame()
        frame_choix_f.setFrameStyle(34)
        titre = QLabel('Filtrage des voies avec un filtre rectangulaire')
        self.frequence = QLineEdit('1000', frame_choix_f)
        hertz = QLabel('Hz', frame_choix_f)
        self.Bexecuter = QPushButton('Exécuter', frame_choix_f)
        self.Bexecuter.clicked.connect(self.calculer)
        self.Baide = QPushButton('Aide', frame_choix_f)
        self.Baide.clicked.connect(self.lance_aide)

        # a figure instance to plot on
        self.figureO = plt.figure()
        self.figureO.patch.set_facecolor('white')

        # this is the Canvas Widget that displays the `figure`
        # it takes the `figure` instance as a parameter to __init__
        self.canvasO = FigureCanvas(self.figureO)
        # create an axis
        self.axCH1 = self.figureO.add_subplot(211)
        self.axCH2 = self.axCH1.twinx()

        # create an other axis
        self.axCH1_f = self.figureO.add_subplot(212)
        self.axCH2_f = self.axCH1_f.twinx()

        # this is the Navigation widget
        # it takes the Canvas widget and a parent
        self.toolbarO = NavigationToolbar(self.canvasO, self)

        #  bouton copier
        bouton_copier = QPushButton('Copier')
        bouton_copier.clicked.connect(self.copier)

        #  remplacer une voie par le résultat
        frame_remplacer = QFrame()
        titre_r = QLabel('Remplacer, dans la page principale, une voie par le signal filtré :')
        self.RB_CH1 = QRadioButton('CH1', frame_remplacer)
        self.RB_CH1.setAutoExclusive(False)
        self.RB_CH2 = QRadioButton('CH2', frame_remplacer)
        self.RB_CH2.setAutoExclusive(False)
        self.RB_lesdeux = QRadioButton('Les deux voies', frame_remplacer)
        self.RB_lesdeux.setAutoExclusive(False)
        self.RB_CH1.toggled.connect(self.remplacer)
        self.RB_CH2.toggled.connect(self.remplacer)
        self.RB_lesdeux.toggled.connect(self.remplacer)

        bouton_restituer = QPushButton("Restituer les voies d'origine")
        bouton_restituer.clicked.connect(self.restituer)

        layout = QVBoxLayout()

        layout_groupe = QVBoxLayout()
        layout_groupe.addWidget(titre)
        layoutFormule = QHBoxLayout()
        layoutFormule.addWidget(self.frequence)
        layoutFormule.addWidget(hertz)
        layoutFormule.addWidget(self.Bexecuter)
        layoutFormule.addWidget(self.Baide)
        layoutFormule.addStretch(1)

        layout_groupe.addLayout(layoutFormule)
        frame_choix_f.setLayout(layout_groupe)

        layoutGraph = QVBoxLayout()  # contient le graphe et ses menus
        layoutGraph.addWidget(self.canvasO)
        layoutGraph.addWidget(self.toolbarO)

        layoutCopier = QHBoxLayout()
        layoutCopier.addStretch(1)
        layoutCopier.addWidget(bouton_copier)
        layoutCopier.addStretch(1)

        layout_remplacer = QHBoxLayout()
        layout_remplacer.addWidget(titre_r)
        layout_remplacer.addWidget(self.RB_CH1)
        layout_remplacer.addWidget(self.RB_CH2)
        layout_remplacer.addWidget(self.RB_lesdeux)
        frame_remplacer.setLayout(layout_remplacer)

        layout.addWidget(frame_choix_f)
        layout.addLayout(layoutGraph)
        layout.addLayout(layoutCopier)
        layout.addWidget(frame_remplacer)
        layout.addWidget(bouton_restituer)

        self.setLayout(layout)

        self.RB_lesdeux.setChecked(False)
        self.RB_CH1.setChecked(False)
        self.RB_CH2.setChecked(False)
        self.CH1_f = []
        self.CH2_f = []
        self.titre1 = 'Résultat'
        self.rep_filtre = []
        self.calculer()
        self.show()

    def copier(self):
        screen = QGuiApplication.primaryScreen()
        p = screen.grabWindow(self.winId())
        pressPaper = QApplication.clipboard()
        pressPaper.setImage(p.toImage())

    def calculer(self):
        try:
            #  calcul de la fréquence de coupure ramenée à la période d'échantillonage
            fl = float(self.frequence.text()) * main.Te
            N = int(1/2/fl) + 59 # nombre de points de la réponse impulsionnelle
            if N % 2 == 0: N += 1  # N doit être impair
            # Compute sinc filter. (réponse impulsionnelle d'un filtre rectangulaire)
            self.rep_filtre = sinc(2 * fl * (arange(N) - (N - 1) / 2))
            # Apply window.
            self.rep_filtre *= blackman(N)
            # Normalize to get unity gain.
            self.rep_filtre /= sum(self.rep_filtre)

            self.CH1_f = convolve(main.voie1, self.rep_filtre)
            self.CH2_f = convolve(main.voie2, self.rep_filtre)

            self.plotO()
        except ZeroDivisionError:
            self.message('Elle doit être >=0.')
        except ValueError:
            self.message('Entrez un nombre positif.')

    def lance_aide(self):
        main.lance_aide("/aide/aide-filtrage.html")

    def message(self, e):
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Critical)

        msg.setText("La fréquence de coupure n'est pas valide.")
        msg.setWindowTitle("Erreur")
        msg.setInformativeText(e)
        msg.setStandardButtons(QMessageBox.Ok)
        msg.setDefaultButton(QMessageBox.Ok)
        msg.exec()

    def remplacer(self):
        if self.RB_CH1.isChecked() or self.RB_lesdeux.isChecked():
            main.voie1 = self.CH1_f

        if self.RB_CH2.isChecked() or self.RB_lesdeux.isChecked():
            main.voie2 = self.CH2_f

        main.mesures()
        main.plot()
        self.courbes_changees()

    def restituer(self):
        main.voie2 = main.voie2_old
        main.voie1 = main.voie1_old

        #  dé-cocher les boutons
        self.RB_lesdeux.setChecked(False)
        self.RB_CH1.setChecked(False)
        self.RB_CH2.setChecked(False)

        main.mesures()
        main.plot()
        self.plotO()
        self.courbes_changees()

    def courbes_changees(self):
        #  rafraichit les autres pages si les signaux ont changé
        if main.spectres_ouvert: main.spectres()
        if main.operations_ouvert: main.fen_operations.calculer()

    def plotO(self):
        # discards the old graph
        self.axCH1.clear()
        self.axCH2.clear()
        self.axCH1_f.clear()
        self.axCH2_f.clear()
        Tt = arange(main.n) * main.Te
        nulle = linspace(0.0, 0.0, num=main.n)
        ntmax = main.n - 1

        # plot data

        self.axCH1.plot(Tt[:ntmax], nulle[:ntmax], 'Maroon', linewidth=1)
        self.axCH1.plot(Tt[:ntmax], main.voie1[:ntmax], 'red', linewidth=1, label='CH1')
        self.axCH2.plot(Tt[:ntmax], nulle[:ntmax], 'Navy', linewidth=1)
        self.axCH2.plot(Tt[:ntmax], main.voie2[:ntmax], 'blue', linewidth=1, label='CH2')

        self.axCH1_f.plot(Tt[:ntmax], nulle[:ntmax], 'black', linewidth=1)
        self.axCH1_f.plot(Tt[:ntmax], self.CH1_f[:ntmax], 'red', linewidth=1, label='CH1 filtré')
        self.axCH2_f.plot(Tt[:ntmax], nulle[:ntmax], 'black', linewidth=1)
        self.axCH2_f.plot(Tt[:ntmax], self.CH2_f[:ntmax], 'Navy', linewidth=1, label='CH2 filtré')

        #self.axCH1.set_ylim([(-127 - main.Yoff1) * main.Ymul1, (127 - main.Yoff1) * main.Ymul1])
        #self.axCH2.set_ylim([(-127 - main.Yoff2) * main.Ymul2, (127 - main.Yoff2) * main.Ymul2])

        # légendes
        self.axCH1.set_ylabel("CH1 (V)", color='red')
        #  self.axCH2.set_ylabel("CH2 (V)")
        self.axCH1.set_xlabel("temps (s)")
        self.axCH1_f.set_ylabel('CH1 filtré', color='red')
        #  self.axCH2_f.set_ylabel('CH2 filtré')

        self.axCH1_f.set_xlabel("temps (s)")
        self.axCH1.grid(True)
        self.axCH1_f.grid(True)

        # refresh canvas
        self.canvasO.draw()

    def keyPressEvent(self, event):
        #  print('keypressEvent')
        #  réimplémente la méthode pour passer en manu en tapant entrée
        if type(event) == PyQt5.QtGui.QKeyEvent:
            if event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter:
                if self.frequence.hasFocus():
                    self.calculer()

    def closeEvent(self, QCloseEvent):
        main.filtrage_ouvert = False


class PageSpectre(QWidget):
    def __init__(self, **kwargs):
        QWidget.__init__(self)

        self.setWindowTitle("Calcul des spectres des voies")

        # choix pour FFT affinée
        frame_aff = QFrame()
        frame_aff.setFrameStyle(34)
        frame_aff.setLineWidth(2)
        frame_aff.setMidLineWidth(1)
        label_aff = QLabel(frame_aff)
        label_aff.setText("Affinage du spectre : entrez la fréquence du signal")
        self.SB_f = QDoubleSpinBox(frame_aff)
        self.SB_f.setMaximum(1e7)
        self.SB_f.setMinimum(0.00001)
        self.SB_f.setValue(50.0)
        self.SB_f.setSuffix(' Hz')
        #  self.SB_f.setStepType(1)  #  permet d'entrer des valeurs décimales au clavier
        self.RB_aff = QRadioButton("Affiner", frame_aff)
        self.RB_aff.toggled.connect(self.affine)
        bouton_aide = QPushButton('Aide', frame_aff)
        bouton_aide.clicked.connect(self.lance_aide)
        self.message = QLabel(' ')
        self.message.setStyleSheet("QLabel {color : 'Red';font-size : 14px;}")
        #  choix fmax
        frame_fmax = QFrame()
        frame_fmax.setFrameStyle(34)
        frame_fmax.setLineWidth(2)
        frame_fmax.setMidLineWidth(1)
        label_fmax = QLabel('Fréquence maximale affichée : ')
        self.RB_fmax_max = QRadioButton('maximale')
        self.RB_fmax_max.toggled.connect(self.plot)
        self.RB_fmax_auto = QRadioButton('auto')
        self.RB_fmax_auto.toggled.connect(self.plot)
        self.RB_fmax_manu = QRadioButton('manuelle :')
        self.RB_fmax_manu.toggled.connect(self.plot)
        self.SB_fmax = QDoubleSpinBox()
        self.SB_fmax.setMaximum(1e7)
        self.SB_fmax.setMinimum(0.1)
        self.SB_fmax.setValue(1000.0)
        self.SB_fmax.setSuffix(' Hz')

        # a figure instance to plot on
        self.figure1 = plt.figure()
        self.figure1.patch.set_facecolor('white')

        # this is the Canvas Widget that displays the `figure`
        # it takes the `figure` instance as a parameter to __init__
        self.canvas1 = FigureCanvas(self.figure1)
        # create an axis
        self.axFFTCH1 = self.figure1.add_subplot(111)
        self.axFFTCH2 = self.axFFTCH1.twinx()

        # this is the Navigation widget
        # it takes the Canvas widget and a parent
        self.toolbar1 = NavigationToolbar(self.canvas1, self)

        #  curseurs
        self.couleur1 = 'Maroon'
        self.couleur2 = 'Navy'
        self.label_cur_1 = QLabel('Curseur 1')
        self.label_cur_1.setStyleSheet("QLabel {color : 'Maroon';font-size : 14px;}")
        self.voie_cur1 = QComboBox()
        self.voie_cur1.addItems(['FFT1', 'FFT2'])
        self.voie_cur1.currentIndexChanged.connect(self.voie_cur)
        self.voie_cur2 = QComboBox()
        self.voie_cur2.addItems(['FFT1', 'FFT2'])
        self.voie_cur2.setCurrentIndex(1)
        self.voie_cur2.currentIndexChanged.connect(self.voie_cur)

        self.curseur1 = QSlider(Qt.Horizontal)
        self.curseur1.valueChanged.connect(self.plot)

        self.cur1_t = QLabel('f1 = ---')
        self.cur1_t.setStyleSheet("QLabel {color : 'Maroon';font-size : 14px;}")
        self.cur1_v = QLabel('v1 = ---')
        self.cur1_v.setStyleSheet("QLabel {color : 'Maroon';font-size : 14px;}")

        self.label_cur_2 = QLabel('Curseur 2')
        self.label_cur_2.setStyleSheet("QLabel {color : 'Navy';font-size : 14px;}")

        self.curseur2 = QSlider(Qt.Horizontal)
        self.curseur2.valueChanged.connect(self.plot)

        self.cur2_t = QLabel('f2 = --- s')
        self.cur2_t.setStyleSheet("QLabel {color : 'Navy';font-size : 14px;}")
        self.cur2_v = QLabel('v2 = --- V')
        self.cur2_v.setStyleSheet("QLabel {color : 'Navy';font-size : 14px;}")

        frame_zoom = QFrame()
        self.RB_zoom = QRadioButton('Zoom entre les curseurs', frame_zoom)
        self.RB_zoom.toggled.connect(self.zoom)

        #  choix affichage
        self.grpechoix = QGroupBox("Affichage des voies")
        self.boutondeux = QRadioButton("Les deux")
        self.boutonch1 = QRadioButton("CH1")
        self.boutonch2 = QRadioButton("CH2")
        self.boutondeux.toggled.connect(self.choix_voies_affichees)
        self.boutonch1.toggled.connect(self.choix_voies_affichees)
        self.boutonch2.toggled.connect(self.choix_voies_affichees)
        lab_CH1 = QLabel('CH1')
        lab_CH1.setStyleSheet("""
                        .QLabel {
                            color : 'Red';
                            font-size: 18px;
                            font-weight: bold;
                            }
                        """)
        lab_CH2 = QLabel('CH2')
        lab_CH2.setStyleSheet("""
                                .QLabel {
                                    color : 'Blue';
                                    font-size: 18px;
                                    font-weight: bold;
                                    }
                                """)
        bouton_copier = QPushButton('Copier')
        bouton_copier.clicked.connect(self.copier)

        # set the layout______________________________________________________________________________________
        layout = QVBoxLayout()  # layout principal

        layoutAff = QHBoxLayout()
        layoutAff.addWidget(label_aff)
        layoutAff.addWidget(self.SB_f)
        layoutAff.addWidget(self.RB_aff)
        layoutAff.addWidget(bouton_aide)
        layoutAff.addWidget(self.message)
        layoutAff.addStretch(1)
        frame_aff.setLayout(layoutAff)

        layout_fmax = QHBoxLayout()
        layout_fmax.addWidget(label_fmax)
        layout_fmax.addWidget(self.RB_fmax_max)
        layout_fmax.addWidget(self.RB_fmax_auto)
        layout_fmax.addWidget(self.RB_fmax_manu)
        layout_fmax.addWidget(self.SB_fmax)
        layout_fmax.addStretch(1)
        frame_fmax.setLayout(layout_fmax)

        layoutGraph = QVBoxLayout()  # contient le graphe et ses menus
        layoutGraph.addWidget(self.canvas1)
        layoutGraph.addWidget(self.toolbar1)

        layout_choix_voies = QVBoxLayout()
        layout_choix_voies.addWidget(self.boutondeux)
        layout_choix_voies.addWidget(self.boutonch1)
        layout_choix_voies.addWidget(self.boutonch2)
        layout_choix_voies.addStretch(1)
        layout_choix_voies.addWidget(lab_CH1)
        layout_choix_voies.addWidget(lab_CH2)
        layout_choix_voies.addStretch(1)
        layout_choix_voies.addWidget(bouton_copier)
        layout_choix_voies.addStretch(1)
        self.grpechoix.setLayout(layout_choix_voies)

        layoutGraphetVoies = QHBoxLayout()
        layoutGraphetVoies.addLayout(layoutGraph)
        layoutGraphetVoies.addWidget(self.grpechoix)

        layout_curseurs = QGridLayout()
        layout_curseurs.addWidget(self.label_cur_1, 0, 0)
        layout_curseurs.addWidget(self.voie_cur1, 0, 1)
        layout_curseurs.addWidget(self.curseur1, 0, 2)
        layout_curseurs.addWidget(self.cur1_t, 0, 3)
        layout_curseurs.addWidget(self.cur1_v, 0, 4)
        layout_curseurs.addWidget(self.label_cur_2, 1, 0)
        layout_curseurs.addWidget(self.voie_cur2, 1, 1)
        layout_curseurs.addWidget(self.curseur2, 1, 2)
        layout_curseurs.addWidget(self.cur2_t, 1, 3)
        layout_curseurs.addWidget(self.cur2_v, 1, 4)

        layout_zoom = QHBoxLayout()
        layout_zoom.addWidget(self.RB_zoom)
        frame_zoom.setLayout(layout_zoom)
        layout_curseurs.addWidget(frame_zoom, 2, 2)

        layout.addWidget(frame_aff)
        layout.addWidget(frame_fmax)
        layout.addLayout(layoutGraphetVoies)
        layout.addLayout(layout_curseurs)

        self.setLayout(layout)
        self.initialisations()
        self.show()

        # ____________________________________________________________________________________________________

    def copier(self):
        screen = QGuiApplication.primaryScreen()
        p = screen.grabWindow(self.winId())
        pressPaper = QApplication.clipboard()
        pressPaper.setImage(p.toImage())

    def initialisations(self):
        # ________________________________________________________initialisations____________________________________________
        self.n = main.n  # nombre de points d'échantillonage
        self.curseur1.setMaximum(self.n-1)
        self.curseur2.setMaximum(self.n-1)
        self.Te = main.Te
        self.voie1 = main.voie1
        #  self.voie1_old = [0.0] * 2500
        self.voie1_aff = []  # signal ré-échantillonné pour la FFT affinée
        self.voie2 = main.voie2
        #  self.voie2_old = [0.0] * 2500
        self.voie2_aff = []  # signal ré-échantillonné pour la FFT affinée
        self.ntmax = main.ntmax
        self.ntmax_old = main.ntmax_old
        self.ntmin = main.ntmin
        self.nFmin = 0
        self.nFmax = 1250
        self.nFmax_old = 1250
        self.f1 = main.f1
        self.f2 = main.f2
        self.flag_aff = main.flag_aff
        self.aff_ch1 = main.aff_ch1
        self.aff_ch2 = main.aff_ch2
        self.plottedonc = True
        self.pasdesignal = main.pasdesignal
        self.erreur_de_freq = False
        self.boutondeux.setChecked(True)
        self.RB_fmax_auto.setChecked(True)
        self.RB_aff.setChecked(False)
        self.RB_zoom.setChecked(False)

    def lance_aide(self):
        self.newbrowser = None
        self.newbrowser = aide("/aide/aide-FFT.html")

    def voie_cur(self, i):
        #  print('voie_curseur')
        liste_couleurs = ['Maroon', 'Navy']

        self.couleur1 = liste_couleurs[self.voie_cur1.currentIndex()]
        style = "QLabel {color : " + self.couleur1 + ";}"
        self.label_cur_1.setStyleSheet(style)
        self.cur1_t.setStyleSheet(style)
        self.cur1_v.setStyleSheet(style)

        self.couleur2 = liste_couleurs[self.voie_cur2.currentIndex()]
        style = "QLabel {color : " + self.couleur2 + ";}"
        self.label_cur_2.setStyleSheet(style)
        self.cur2_t.setStyleSheet(style)
        self.cur2_v.setStyleSheet(style)

        self.curseur1.setMaximum(int(self.n / 4.0) - 1)
        self.curseur1.setMinimum(0)

        self.curseur2.setMaximum(int(self.n / 4.0) - 1)
        self.curseur2.setMinimum(0)

        self.plot()

    def choix_voies_affichees(self):
        #  print('choix_voies_affichees')
        if self.boutondeux.isChecked():
            self.aff_ch1 = True
            self.aff_ch2 = True
        else:
            if self.boutonch1.isChecked():
                self.aff_ch1 = True
                self.aff_ch2 = False
            else:
                if self.boutonch2.isChecked():
                    self.aff_ch1 = False
                    self.aff_ch2 = True
        if self.plottedonc: self.plot()

    def max_curseurs(self, nfm):
        self.curseur1.setMaximum(nfm)
        self.curseur2.setMaximum(nfm)

    def zoom(self):
        #  print('zoom')
        if self.RB_zoom.isChecked():
            if self.curseur1.value() < self.curseur2.value():
                self.nFmax_old = self.nFmax
                self.nFmin = self.curseur1.value()
                self.nFmax = self.curseur2.value()
            elif self.curseur1.value() > self.curseur2.value():
                self.nFmax_old = self.nFmax
                self.nFmax = self.curseur1.value()
                self.nFmin = self.curseur2.value()
        else:
            self.nFmax = self.nFmax_old
            self.nFmin = 0
        self.plot()


    def reech(self, voie):
        #  print('reech')
        #  ré-échantillonnage sur 1 période
        retour = []
        self.Te_aff = 1.0 / self.SB_f.value() / 2048.0
        try:
            for i in range(2048):
                ti = i * self.Te_aff  # abscisse
                j = int(ti / self.Te)  # n° du point juste avant l'abscisse
                an = (voie[j + 1] - voie[j]) / self.Te
                bn = voie[j] - an * j * self.Te
                retour.append(an * ti + bn)

            #  duplication 16 fois de la période
            for i in range(4):
                retour = retour + retour
            return True, retour
        except IndexError:
            self.erreur_de_freq = True
            self.message.setText('La fréquence donnée est incorrecte !')
            return False, voie

    def affine(self):
        #  print('affine')
        """" prépare le signal pour affiner la fft"""
        if not self.pasdesignal:
            if self.RB_aff.isChecked():
                OK, self.voie1_aff = self.reech(self.voie1)
                OK, self.voie2_aff = self.reech(self.voie2)
                if OK:
                    self.n = len(self.voie1_aff)
                    self.flag_aff = True
                else:
                    self.flag_aff = False
                    self.RB_aff.setChecked(False)
            else:
                self.flag_aff = False
                self.n = 2500

            self.plot()

    def keyPressEvent(self, event):
        #  print('keypressEvent')
        #  réimplémente la méthode pour passer en manu en tapant entrée
        if type(event) == PyQt5.QtGui.QKeyEvent:
            if event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter:
                if self.SB_fmax.hasFocus():
                    if self.RB_fmax_manu.isChecked():
                        self.plot()
                    else:
                        self.RB_fmax_manu.setChecked(True)
                if self.SB_f.hasFocus():
                    self.message.setText('')
                    if self.RB_aff.isChecked():
                        self.plot()
                    else:
                        self.RB_aff.setChecked(True)

    def plot(self):
        if self.plottedonc:
            #  print('plot')
            #  choisir les variables si affinée
            if self.flag_aff:
                v1 = self.voie1_aff
                v2 = self.voie2_aff
                Te = self.Te_aff
                if not self.RB_zoom.isChecked():
                    self.ntmax = 2 * int(1 / self.SB_f.value() / Te - 1)
            else:
                v1 = self.voie1
                v2 = self.voie2
                Te = self.Te
                if not self.RB_zoom.isChecked():
                    self.ntmax = self.n - 1
            Tt = arange(self.n) * Te
            #  spectres
            freq = fft.fftfreq(self.n, Te)
            if self.aff_ch1:
                sp1 = 2 * scfft(v1, int(self.n), axis=0) / self.n
                sp1[0] = sp1[0] / 2  # pour obtenir la bonne amplitude de la raie continue qui n'est pas dédoublée comme les autres
                y1 = abs(sp1)  # calcul du module
            if self.aff_ch2:
                sp2 = 2 * scfft(v2, int(self.n), axis=0) / self.n
                sp2[0] = sp2[0] / 2  # pour obtenir la bonne amplitude de la raie continue qui n'est pas dédoublée comme les autres
                y2 = abs(sp2)  # calcul du module

            #  gestion de fmax
            if not self.RB_zoom.isChecked():
                self.nFmax = int(len(freq) / 4)
                if self.RB_fmax_manu.isChecked():
                    self.nFmax = int(self.SB_fmax.value() / freq[1])
                if self.RB_fmax_auto.isChecked():
                    i = 0
                    j = 0
                    if self.aff_ch1:
                        max1 = 0.0
                        for item in y1:
                            if max1 < item: max1 = item
                        i = int(len(freq) / 4)
                        while (y1[i] < max1 / 50) and (i > 2):
                            i -= 1
                    if self.aff_ch2:
                        max2 = 0.0
                        for item in y2:
                            if max2 < item: max2 = item
                        j = int(len(freq) / 4)
                        while (y2[j] < max2 / 50) and (j > 2):  #  on cherche la dernière raie >max/50
                            j -= 1
                    if i > j:
                        self.nFmax = int(1.5*i)
                    else:
                        self.nFmax = int(1.5*j)
                if self.RB_fmax_max.isChecked() or self.nFmax > int(len(freq) / 4):  #  limite aussi les débordements
                    self.nFmax = int(len(freq) / 4)

                self.max_curseurs(self.nFmax)

            # courbe nulle pour marquer l'axe
            nulle = linspace(0.0, 0.0, num=self.n)

            # discards the old graph
            self.axFFTCH1.clear()
            self.axFFTCH2.clear()

            # plot data
            if self.aff_ch1:
                self.axFFTCH1.set_visible(True)
                self.axFFTCH1.plot(freq[0:self.nFmax], y1[0:self.nFmax], 'red', linewidth=1)
                self.axFFTCH1.plot(freq[0:self.nFmax], nulle[0:self.nFmax], 'black', linewidth=1)
                #  curseur 1
                if self.voie_cur1.currentIndex() == 0:  #  sur FFT1
                    t1 = freq[self.curseur1.value()]
                    ord1 = y1[self.curseur1.value()]
                    self.axFFTCH1.plot(t1, ord1, self.couleur1, marker=">")
                    self.cur1_t.setText('f1 = ' + format(t1, '^9.2E') + 'Hz  ')
                    self.cur1_v.setText('v1 = ' + format(ord1, '^9.2E') + 'V')
                #  curseur 2
                if self.voie_cur2.currentIndex() == 0:  #  sur FFT1
                    t2 = freq[self.curseur2.value()]
                    ord2 = y1[self.curseur2.value()]
                    self.axFFTCH1.plot(t2, ord2, self.couleur1, marker="<")
                    self.cur2_t.setText('f2 = ' + format(t2, '^9.2E') + 'Hz  ')
                    self.cur2_v.setText('v2 = ' + format(ord2, '^9.2E') + 'V')

                if not self.aff_ch2:
                    self.axFFTCH2.plot(freq[0:self.nFmax], y1[0:self.nFmax], 'red', linewidth=1)  # pour que l'échelle verticale soit OK
                    self.axFFTCH2.set_visible(False)  #  mais il y a des décalages de courbes donc on le cache

                self.axFFTCH1.set_ylabel("crête CH1 (V)", color='red')

            if self.aff_ch2:
                self.axFFTCH2.set_visible(True)
                self.axFFTCH2.plot(freq[0:self.nFmax], y2[0:self.nFmax], 'blue', linewidth=1)
                self.axFFTCH2.plot(freq[0:self.nFmax], nulle[0:self.nFmax], 'black', linewidth=1)
                #  curseur 1
                if self.voie_cur1.currentIndex() == 1:  #  sur FFT2
                    t1 = freq[self.curseur1.value()]
                    ord1 = y2[self.curseur1.value()]
                    self.axFFTCH2.plot(t1, ord1, self.couleur2, marker=">")
                    self.cur1_t.setText('f1 = ' + format(t1, '^9.2E') + 'Hz  ')
                    self.cur1_v.setText('v1 = ' + format(ord1, '^9.2E') + 'V')
                #  curseur 2
                if self.voie_cur2.currentIndex() == 1:  #  sur FFT2
                    t2 = freq[self.curseur2.value()]
                    ord2 = y2[self.curseur2.value()]
                    self.axFFTCH2.plot(t2, ord2, self.couleur2, marker="<")
                    self.cur2_t.setText('f2 = ' + format(t2, '^9.2E') + 'Hz  ')
                    self.cur2_v.setText('v2 = ' + format(ord2, '^9.2E') + 'V')

                if not self.aff_ch1:
                    self.axFFTCH1.plot(freq[0:self.nFmax], y2[0:self.nFmax], 'blue', linewidth=1)  # pour que l'échelle verticale soit OK

                #  self.axFFTCH2.set_ylabel("amplitude CH2 (V)")

            self.axFFTCH1.set_xlabel("fréquence (Hz)")
            self.axFFTCH1.grid(True)

            # refresh canvas
            self.canvas1.draw()

    def closeEvent(self, QCloseEvent):
        main.spectres_ouvert = False


class Principale(QWidget):
    def __init__(self, parent=None):
        super(Principale, self).__init__(parent)
        self.setWindowFlags(Qt.WindowFlags())
        self.setWindowTitle("Logiciel pour oscilloscope - P. Philip 2023 - Version 3.2")
        # barre de menus
        self.fct = "Fichier"  #   initialise la variable

        self.menu = QMenuBar()
        fichier = self.menu.addMenu("Fichier")
        fichier.addAction("Ouvrir")  # une ligne de menu sans raccourci.
        fichier.addAction('Enregistrer')
        fichier.triggered[QAction].connect(self.processtrigger)

        math = self.menu.addMenu("Math")
        math.addAction('Opérations sur les voies')
        math.addAction('Spectres (FFT)')
        math.addAction('Filtrage')

        math.triggered[QAction].connect(self.processtrigger)

        aide = self.menu.addMenu("Aide")
        aide.addAction("Sommaire de l'aide")
        aide.addAction('Aide sur la FFT (le spectre)')
        aide.addAction('Aide sur les opérations entre voies')
        aide.addAction('Aide sur le filtrage des voies')
        aide.triggered[QAction].connect(self.processtrigger)

        #  choix affichage
        self.grpechoix = QGroupBox("Affichage des voies")
        self.boutondeux = QRadioButton("Les deux")
        self.boutonch1 = QRadioButton("CH1")
        self.boutonch2 = QRadioButton("CH2")
        self.boutondeux.toggled.connect(self.choix_voies_affichees)
        self.boutonch1.toggled.connect(self.choix_voies_affichees)
        self.boutonch2.toggled.connect(self.choix_voies_affichees)

        lab_CH1 = QLabel('CH1')
        lab_CH1.setStyleSheet("""
                .QLabel {
                    color : 'Red';
                    font-size: 18px;
                    font-weight: bold;
                    }
                """)
        lab_CH2 = QLabel('CH2')
        lab_CH2.setStyleSheet("""
                        .QLabel {
                            color : 'Blue';
                            font-size: 18px;
                            font-weight: bold;
                            }
                        """)
        #  boutons de raccourcis
        self.grpe_raccourcis = QGroupBox("Raccourcis")

        bouton_copier = QPushButton('Copier')
        bouton_copier.clicked.connect(self.copier)

        #  bouton spectres
        bouton_spectres = QPushButton('Spectres')
        bouton_spectres.clicked.connect(self.spectres)

        # a figure instance to plot on
        self.figure1 = plt.figure()
        self.figure1.patch.set_facecolor('white')

        # this is the Canvas Widget that displays the `figure`
        # it takes the `figure` instance as a parameter to __init__
        self.canvas1 = FigureCanvas(self.figure1)
        # create an axis
        self.axtempsCH1 = self.figure1.add_subplot(111)
        self.axtempsCH2 = self.axtempsCH1.twinx()

        # this is the Navigation widget
        # it takes the Canvas widget and a parent
        self.toolbar1 = NavigationToolbar(self.canvas1, self)

        #  curseurs
        self.couleur1 = 'Maroon'
        self.couleur2 = 'Navy'
        self.label_cur_1 = QLabel('Curseur 1')
        self.label_cur_1.setStyleSheet("QLabel {color : 'Maroon';font-size:xx-large;}")
        self.voie_cur1 = QComboBox()
        self.voie_cur1.addItems(['CH1', 'CH2'])
        self.voie_cur1.currentIndexChanged.connect(self.voie_cur)
        self.voie_cur2 = QComboBox()
        self.voie_cur2.addItems(['CH1', 'CH2'])
        self.voie_cur2.setCurrentIndex(1)
        self.voie_cur2.currentIndexChanged.connect(self.voie_cur)

        self.curseur1 = QSlider(Qt.Horizontal)
        self.curseur1.valueChanged.connect(self.plot)

        self.cur1_t = QLabel('t1 = ---')
        self.cur1_t.setStyleSheet("QLabel {color : 'Maroon';font-size : 14px;}")
        self.cur1_v = QLabel('v1 = ---')
        self.cur1_v.setStyleSheet("QLabel {color : 'Maroon';font-size : 14px;}")

        self.label_cur_2 = QLabel('Curseur 2')
        self.label_cur_2.setStyleSheet("QLabel {color : 'Navy';font-size:xx-large;}")

        self.curseur2 = QSlider(Qt.Horizontal)
        self.curseur2.valueChanged.connect(self.plot)

        self.cur2_t = QLabel('t2 = --- s')
        self.cur2_t.setStyleSheet("QLabel {color : 'Navy';font-size : 14px;}")
        self.cur2_v = QLabel('v2 = --- V')
        self.cur2_v.setStyleSheet("QLabel {color : 'Navy';font-size : 14px;}")

        style = "QLabel {font-size:14px;}"
        self.dx = QLabel('Δt = --- s')
        self.dx.setStyleSheet(style)
        self.dv = QLabel('Δv = --- V')
        self.dv.setStyleSheet(style)
        self.f_mes = QLabel('f = --- Hz')
        self.f_mes.setStyleSheet(style)
        self.label_phi = QLabel('φ = --- °')
        self.label_phi.setStyleSheet(style)
        
        frame_zoom = QFrame()
        self.RB_zoom = QRadioButton('Zoom entre les curseurs', frame_zoom)
        self.RB_zoom.toggled.connect(self.zoom)
        self.label_integrale = QLabel('Intégrale entre les curseurs : --- Vs')
        self.label_integrale.setStyleSheet(style)

        #  mesures
        frame_mesures = QFrame()
        frame_mesures.setFrameStyle(34)
        frame_mesures.setLineWidth(2)
        frame_mesures.setMidLineWidth(1)
        #  frame_mesures.setStyleSheet("QFrame {background-color: ivory}")
        label_mes1 = QLabel('Mesures sur CH1 : RMS = ', frame_mesures)
        self.rms1 = QLabel('--- V', frame_mesures)
        label_mesf1 = QLabel(' ;     Fréquence calculée = ', frame_mesures)
        self.label_f1 = QLabel('--- Hz', frame_mesures)
        label_mesDC1 = QLabel(' ;    Moyenne = ', frame_mesures)
        self.DC1 = QLabel('--- V', frame_mesures)
        label_mes2 = QLabel('Mesures sur CH2 : RMS = ', frame_mesures)
        self.rms2 = QLabel('--- V', frame_mesures)
        label_mesf2 = QLabel(' ;     Fréquence calculée = ', frame_mesures)
        self.label_f2 = QLabel('--- Hz', frame_mesures)
        label_mesDC2 = QLabel(' ;    Moyenne = ', frame_mesures)
        self.DC2 = QLabel('--- V', frame_mesures)
        style_mes_1 = "QLabel {color : 'red';font-size : 14px;}"
        label_mes1.setStyleSheet(style_mes_1)
        self.rms1.setStyleSheet(style_mes_1)
        label_mesf1.setStyleSheet(style_mes_1)
        self.label_f1.setStyleSheet(style_mes_1)
        label_mesDC1.setStyleSheet(style_mes_1)
        self.DC1.setStyleSheet(style_mes_1)
        style_mes_2 = "QLabel {color : 'blue';font-size : 14px;}"
        label_mes2.setStyleSheet(style_mes_2)
        self.rms2.setStyleSheet(style_mes_2)
        label_mesf2.setStyleSheet(style_mes_2)
        self.label_f2.setStyleSheet(style_mes_2)
        label_mesDC2.setStyleSheet(style_mes_2)
        self.DC2.setStyleSheet(style_mes_2)

        # button connected to `transfert` method
        self.buttonT = QPushButton('Transfert')
        self.buttonT.setFixedWidth(200)
        self.buttonT.clicked.connect(self.transfert)

        # LED à partir d'un label
        self.LED = QLabel("Pas d'oscillo")
        self.LED.setFixedWidth(100)
        self.LED.setFixedHeight(30)
        self.LED.setAlignment(Qt.AlignHCenter)
        self.LED.setAlignment(Qt.AlignVCenter)
        self.LED.setStyleSheet("""
        .QLabel {
            border: 2px solid maroon;
            border-radius: 10px;
            background-color: 'red';
            color : 'yellow';
            }
        """)

        #  label infos diverses
        self.infos = QLabel("")
        self.infos.setFixedHeight(30)
        self.infos.setAlignment(Qt.AlignHCenter)
        self.infos.setAlignment(Qt.AlignVCenter)
        self.infos.setStyleSheet("""
        .QLabel {
            border: 2px solid maroon;
            border-radius: 10px;
            background-color: 'white';
            color : 'blue';
            }
        """)

        # set the layout______________________________________________________________________________________
        layout = QVBoxLayout()  # layout principal

        self.layoutGraph = QVBoxLayout()  # contient le graphe et ses menus
        self.layoutGraph.addWidget(self.canvas1)
        self.layoutGraph.addWidget(self.toolbar1)

        layout_choix_voies = QVBoxLayout()
        layout_choix_voies.addWidget(self.boutondeux)
        layout_choix_voies.addWidget(self.boutonch1)
        layout_choix_voies.addWidget(self.boutonch2)
        layout_choix_voies.addStretch(1)
        
        layout_legendes = QHBoxLayout()
        layout_legendes.addWidget(lab_CH1)
        layout_legendes.addWidget(lab_CH2)
        layout_choix_voies.addLayout(layout_legendes)
        self.grpechoix.setLayout(layout_choix_voies)

        layout_choix_raccourcis = QVBoxLayout()
        layout_choix_raccourcis.addWidget(self.grpechoix)
        layout_choix_raccourcis.addWidget(bouton_copier)
        layout_choix_raccourcis.addStretch(1)
        layout_choix_raccourcis.addWidget(bouton_spectres)
        
        layoutGraphetVoies = QHBoxLayout()

        layoutGraphetVoies.addLayout(self.layoutGraph)
        layoutGraphetVoies.addLayout(layout_choix_raccourcis)

        layout_curseurs = QGridLayout()
        layout_curseurs.addWidget(self.label_cur_1, 0, 0)
        layout_curseurs.addWidget(self.voie_cur1, 0, 1)
        layout_curseurs.addWidget(self.curseur1, 0, 2)
        layout_curseurs.addWidget(self.cur1_t, 0, 3)
        layout_curseurs.addWidget(self.cur1_v, 0, 4)
        layout_curseurs.addWidget(self.label_cur_2, 1, 0)
        layout_curseurs.addWidget(self.voie_cur2, 1, 1)
        layout_curseurs.addWidget(self.curseur2, 1, 2)
        layout_curseurs.addWidget(self.cur2_t, 1, 3)
        layout_curseurs.addWidget(self.cur2_v, 1, 4)
        layout_curseurs.addWidget(self.dx, 2, 3)
        layout_curseurs.addWidget(self.dv, 2, 4)
        layout_zoom = QHBoxLayout()
        layout_zoom.addWidget(self.RB_zoom)
        frame_zoom.setLayout(layout_zoom)
        layout_curseurs.addWidget(frame_zoom, 2, 2)
        layout_curseurs.addWidget(self.label_integrale, 3, 2)
        layout_curseurs.addWidget(self.f_mes, 3, 3)
        layout_curseurs.addWidget(self.label_phi, 3, 4)

        layout_mesures = QVBoxLayout()
        layout_mesures1 = QHBoxLayout()
        layout_mesures1.addWidget(label_mes1)
        layout_mesures1.addWidget(self.rms1)
        layout_mesures1.addWidget(label_mesf1)
        layout_mesures1.addWidget(self.label_f1)
        layout_mesures1.addWidget(label_mesDC1)
        layout_mesures1.addWidget(self.DC1)
        layout_mesures1.addStretch(1)
        layout_mesures2 = QHBoxLayout()
        layout_mesures2.addWidget(label_mes2)
        layout_mesures2.addWidget(self.rms2)
        layout_mesures2.addWidget(label_mesf2)
        layout_mesures2.addWidget(self.label_f2)
        layout_mesures2.addWidget(label_mesDC2)
        layout_mesures2.addWidget(self.DC2)
        layout_mesures2.addStretch(1)
        layout_mesures.addLayout(layout_mesures1)
        layout_mesures.addLayout(layout_mesures2)
        frame_mesures.setLayout(layout_mesures)

        LayoutTransfert = QHBoxLayout()  # contient le bouton Transfert, patientez et détectionOK
        LayoutTransfert.addWidget(self.buttonT)
        LayoutTransfert.addWidget(self.LED)
        LayoutTransfert.addWidget(self.infos)

        layout.addWidget(self.menu)
        layout.addLayout(layoutGraphetVoies)
        layout.addLayout(layout_curseurs)
        layout.addWidget(frame_mesures)
        layout.addLayout(LayoutTransfert)

        self.setLayout(layout)

        # ________________________________________________________initialisations____________________________________________
        self.adresseVisa = 'vide'
        self.n = 2500  # nombre de points d'échantillonage
        self.curseur1.setMaximum(self.n-1)
        self.curseur2.setMaximum(self.n-1)
        self.Te = 0.0001
        self.voie1 = [0.0]*2500
        self.voie1_old = [0.0]*2500
        self.voie1_aff = []  # signal ré-échantillonné pour la FFT affinée
        self.voie2 = [0.0]*2500
        self.voie2_old = [0.0]*2500
        self.voie2_aff = []  # signal ré-échantillonné pour la FFT affinée
        self.Ymul1 = 0.0
        self.Yoff1 = 0.0
        self.Ymul2 = 0.0
        self.Yoff2 = 0.0
        self.ntmax = 2500
        self.ntmax_old = 2500
        self.ntmin = 0
        self.f1 = 0.0
        self.f2 = 0.0
        self.flag_ouvrir = True
        self.flag_aff = False
        self.aff_ch1 = True
        self.aff_ch2 = True
        self.plottedonc = True
        self.mesures()
        self.pasdesignal = True
        self.erreur_de_freq = False
        #  marqueurs de pages ouvertes
        self.spectres_ouvert = False
        self.filtrage_ouvert = False
        self.operations_ouvert = False

        self.type_oscillo = 'aucun'
        self.boutondeux.setChecked(True)
        self.rm = visa.ResourceManager()  #  permet d'utiliser pyvisa-py. Enlever '@py' sous W
        self.trouveVisa()
        # ____________________________________________________________________________________________________

    def copier(self):
        screen = QGuiApplication.primaryScreen()
        p = screen.grabWindow(self.winId())
        pressPaper = QApplication.clipboard()
        pressPaper.setImage(p.toImage())

    def message(self, ligne1, ligne2=''):
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Critical)

        msg.setText(ligne1)
        msg.setWindowTitle("Erreur")
        msg.setInformativeText(ligne2)
        msg.setStandardButtons(QMessageBox.Ok)
        msg.setDefaultButton(QMessageBox.Ok)
        msg.exec()

    def processtrigger(self, q):
        #  print('processtrigger')
        self.fct = q.text()
        if self.fct == "Ouvrir":
            self.ouvrir()

        if self.fct == "Enregistrer":
            self.enregistrer()

        if self.fct == "Opérations sur les voies":
            if self.flag_aff:
                self.message("Cette option n'est pas disponible avec des courbes modifiées par la FFt affinée. Décochez la case 'Affiner' et réessayez.")
            else:
                self.fen_operations = None
                self.operations_ouvert = True
                self.fen_operations = operations()

        if self.fct == 'Spectres (FFT)':
            self.spectres()

        if self.fct == 'Filtrage':
            self.fen_filtrage = None
            self.filtrage_ouvert = True
            self.fen_filtrage = filtrage()

        if self.fct == "Sommaire de l'aide":
            self.lance_aide("/aide/aide.html")

        if self.fct == "Aide sur la FFT (le spectre)":
            self.lance_aide("/aide/aide-FFT.html")

        if self.fct == 'Aide sur les opérations entre voies':
            self.lance_aide("/aide/aide-operations.html")

        if self.fct == 'Aide sur le filtrage des voies':
            self.lance_aide("/aide/aide-filtrage.html")

    def lance_aide(self, url):
        self.newbrowser = None
        self.newbrowser = aide(url)

    def spectres(self):
        if not self.spectres_ouvert:
            self.page_fft = PageSpectre()
            self.spectres_ouvert = True
        else:
            self.page_fft.initialisations()
            self.page_fft.plot()
            self.page_fft.show()

    def choix_voies_affichees(self):
        #  print('choix_voies_affichees')
        if self.boutondeux.isChecked():
            self.aff_ch1 = True
            self.aff_ch2 = True
        else:
            if self.boutonch1.isChecked():
                self.aff_ch1 = True
                self.aff_ch2 = False
            else:
                if self.boutonch2.isChecked():
                    self.aff_ch1 = False
                    self.aff_ch2 = True
        if self.plottedonc: self.plot()

    def voie_cur(self, i):
        #  print('voie_curseur')
        liste_couleurs = ['Maroon', 'Navy', 'Maroon', 'Navy']

        self.couleur1 = liste_couleurs[self.voie_cur1.currentIndex()]
        style = "QLabel {color : " + self.couleur1 + ";}"
        self.label_cur_1.setStyleSheet(style)
        self.cur1_t.setStyleSheet(style)
        self.cur1_v.setStyleSheet(style)

        self.couleur2 = liste_couleurs[self.voie_cur2.currentIndex()]
        style = "QLabel {color : " + self.couleur2 + ";}"
        self.label_cur_2.setStyleSheet(style)
        self.cur2_t.setStyleSheet(style)
        self.cur2_v.setStyleSheet(style)

        self.curseur1.setMaximum(self.ntmax-1)  # penser à changer si aff
        self.curseur1.setMinimum(self.ntmin)

        self.curseur2.setMaximum(self.n-1)  # penser à changer si aff
        self.curseur2.setMinimum(self.ntmin)

        self.RB_zoom.setDisabled(False)

        self.plot()

    def max_curseurs(self, nfm):
        #  print('max_curseurs')
            self.curseur1.setMaximum(self.ntmax-1)
            self.curseur1.setMinimum(self.ntmin)

            self.curseur2.setMaximum(self.ntmax-1)
            self.curseur2.setMinimum(self.ntmin)

    def mesures(self):
        #  print('mesures')
        #  calcul de la longueur d'1 période du signal
        try:
            if self.flag_aff:
                self.f1 = self.SB_f.value()
                self.f2 = self.SB_f.value()
                Te = self.Te_aff
                v1 = self.voie1_aff
                v2 = self.voie2_aff
            else:
                #la fréq n'est pas donnée, on la calcule
                Te = self.Te
                v1 = self.voie1
                v2 = self.voie2
                self.f1 = self.frequence(Te, v1)
                self.f2 = self.frequence(Te, v2)
            #  si f connue, calculer la RMS sur la dernière période pour améliorer la précision
            if self.f1 != 0.0: v1 = v1[(self.n - int(1 / self.f1 / Te)):]
            if self.f2 != 0.0: v2 = v2[(self.n - int(1 / self.f2 / Te)):]
            rms1 = sqrt(mean(square(v1)))
            DC1 = mean(v1)
            rms2 = sqrt(mean(square(v2)))
            DC2 = mean(v2)

            self.rms1.setText(format(rms1, '9.2E') + ' V')
            self.label_f1.setText(format(self.f1, '^9.2E') + ' Hz')
            self.DC1.setText(format(DC1, '^9.2E') + ' V')
            self.rms2.setText(format(rms2, '^9.2E') + ' V')
            self.label_f2.setText(format(self.f2, '^9.2E') + ' Hz')
            self.DC2.setText(format(DC2, '^9.2E') + ' V')
        except:
            self.infos.setText('Mesures impossibles. Les données sur les signaux sont probablement erronées.')

    def zoom(self):
        #  print('zoom')
        if self.RB_zoom.isChecked():
            if self.curseur1.value() < self.curseur2.value():
                self.ntmax_old = self.ntmax
                self.ntmin = self.curseur1.value()
                self.ntmax = self.curseur2.value()
            elif self.curseur1.value() > self.curseur2.value():
                self.ntmax_old = self.ntmax
                self.ntmax = self.curseur1.value()
                self.ntmin = self.curseur2.value()
        else:
            self.ntmax = self.ntmax_old
            self.ntmin = 0
        self.plot()

    def frequence(self, Te, signal):
        #  code from chat gpt, amélioré
        #  estimation de la fréquence de base f0 avec la FFT
        # Calculez la STFT du signal
        n = len(signal)
        frequencies = fft.fftfreq(n, Te)
        stft = fft.fft(signal)

        # Trouvez le pic de fréquence dominante
        magnitude = abs(stft)
        peaks, _ = find_peaks(magnitude, height=0.1)  # Ajustez la hauteur du pic selon votre besoin

        if len(peaks) > 0:
            dominant_frequency = frequencies[peaks[0]]
        else:
            dominant_frequency = 0.0  # Aucun pic trouvé

        #  filtrage PB au-dessus de 10*f0

        #  calcul de la fréquence de coupure ramenée à la période d'échantillonage
        fl = dominant_frequency * 10.0 * Te
        N = int(1 / 2 / fl) + 59  # nombre de points de la réponse impulsionnelle
        if N % 2 == 0: N += 1  # N doit être impair
        # Compute sinc filter. (réponse impulsionnelle d'un filtre rectangulaire)
        rep_filtre = sinc(2 * fl * (arange(N) - (N - 1) / 2))
        # Apply window.
        rep_filtre *= blackman(N)
        # Normalize to get unity gain.
        rep_filtre /= sum(rep_filtre)

        signal_f = convolve(signal, rep_filtre)

        # Calcul de l'autocorrélation du signal filtré => insensible à la porteuse de MLI et + précis que FFT seule
        autocorr = correlate(signal_f, signal_f, mode='full')

        # Seules les valeurs positives de l'autocorrélation sont pertinentes
        autocorr = autocorr[len(autocorr) // 2:]

        # Trouvez l'emplacement du premier pic (à l'exception de zéro)
        peaks, _ = find_peaks(autocorr, height=0.1)  # Ajustez la hauteur du pic selon votre besoin

        if len(peaks) > 0:
            # Trouvez la période en échantillons (nombre d'échantillons par cycle)
            period_samples = peaks[0]

            # Calculez la fréquence en Hz
            dominant_frequency = 1.0 / (period_samples * Te)
        else:
            dominant_frequency = 0.0  # Aucun pic trouvé

        return dominant_frequency
    def ouvrir(self):
        #  print('ouvrir')
        listeFichier = QFileDialog.getOpenFileName(self, "ouvrir un fichier de courbes",
                                                   "c:/SAD/oscillo/data",
                                                   "csv Files (*.csv)")
        # traitement seulement si on a choisi un fichier
        if listeFichier[0] != '':
            self.infos.setText(listeFichier[0])
            v1 = []  #  variables pour les voies , basculement si tout va bien à la fin
            v2 = []
            Te = 0.0
            n = 2500
            try:
                with open(listeFichier[0], newline='') as csvfile:
                    reader = csv.reader(csvfile, delimiter=';')
                    first = True
                    for row in reader:
                        if first:
                            Te = float(row[0])
                            n = int(row[1])
                            first = False
                        else:
                            v1.append(float(row[0]))
                            v2.append(float(row[1]))
                self.Te = Te
                self.n = n
                self.voie1 = v1
                self.voie1_old = v1
                self.voie2 = v2
                self.voie2_old = v2

                self.infos.setText(listeFichier[0])
                self.infos.repaint()
                self.flag_ouvrir = True
                self.curseur1.setMaximum(self.n-1)
                self.curseur2.setMaximum(self.n-1)

                self.mesures()

                self.pasdesignal = False
                self.plottedonc = True
                self.courbes_changees()
                self.plot()
            except:
                self.infos.setText('problème de lecture')
                self.infos.repaint()
                self.message("La lecture du fichier s'est mal passée", "Le fichier ne peut être lu ou bien les données ne sont pas exploitables")

    def enregistrer(self):
        listeFichier = QFileDialog.getSaveFileName(self, "enregistrer un fichier de courbes",
                                                   "c:/SAD/oscillo/data",
                                                   "csv Files (*.csv)")
        # traitement seulement si on a choisi un fichier
        if listeFichier[0] != '':
            self.infos.setText(listeFichier[0])
            with open(listeFichier[0], 'w', newline='') as csvfile:
                writer = csv.writer(csvfile, delimiter=';',
                                    quotechar='|', quoting=csv.QUOTE_MINIMAL)
                writer.writerow([self.Te, 2500])
                i = 0
                for item in self.voie1:
                    writer.writerow([item, self.voie2[i]])  # chaque ligne contient voie1(t) et voie2(t)
                    i = i + 1

    def transfert(self):
        self.infos.setText("transfert en cours")
        self.infos.repaint()

        try:
            self.LED.setText("Transfert...")
            self.LED.setStyleSheet("""
                        .QLabel {
                            border: 2px solid maroon;
                            border-radius: 10px;
                            background-color: #ef7b0a;
                            color : 'yellow';
                            }
                        """)
            self.oscillo = self.rm.open_resource(self.adresseVisa)
            self.infos.setText('Transfert sur' + self.adresseVisa)
            self.infos.repaint()
            # initialisations de l'oscillo___________________
            self.oscillo.write("HEAD 0")
            self.oscillo.timeout = 15000
            self.oscillo.write("DAT:ENC ASCI")
            self.oscillo.write("DAT:WID 1")
    
            #  test voies____________________________________
            voies_valides = self.oscillo.query("SEL?")
            if self.type_oscillo == 'TDS':
                l_voies = list(voies_valides)
                ch1 = (l_voies[0] == '1')
                ch2 = (l_voies[2] == '1')
    
            if self.type_oscillo == 'TBS':
                l_voies = list(voies_valides)
                ch1 = (l_voies[2] == '1')
                ch2 = (l_voies[4] == '1')
    
            transferer = ch1 or ch2
            # _______________________________________________________
            if transferer:
                self.boutondeux.setEnabled(True)
                self.RB_zoom.setChecked(False)
                self.aff_ch1 = True
                self.aff_ch2 = True
                if ch1:
                    # transfert CH1
                    self.oscillo.write("DAT:SOU CH1")
                    self.Te = float(self.oscillo.query("WFMP:XIN?"))
                    Ymul1 = float(self.oscillo.query("WFMP:YMU?"))
                    Yoff1 = float(self.oscillo.query("WFMP:YOF?"))
                    voie1 = self.oscillo.query_ascii_values("CURV?")
                    for i in range(0, 2499):
                        voie1[i] = (voie1[i] - Yoff1) * Ymul1
                    if not ch2:  # dupliquer la voie 1 sur la 2
                        Ymul2 = Ymul1
                        Yoff2 = Yoff1
                        voie2 = voie1
    
                if ch2:
                    # transfert CH2
                    self.oscillo.write("DAT:SOU CH2")
                    self.Te = float(self.oscillo.query("WFMP:XIN?"))
                    Ymul2 = float(self.oscillo.query("WFMP:YMU?"))
                    Yoff2 = float(self.oscillo.query("WFMP:YOF?"))
                    voie2 = self.oscillo.query_ascii_values("CURV?")
                    for i in range(0, 2499):
                        voie2[i] = (voie2[i] - Yoff2) * Ymul2
                    if not ch1:  # dupliquer la voie 2 sur la 1
                        Ymul1 = Ymul2
                        Yoff1 = Yoff2
                        voie1 = voie2
    
                #  le transfert s'est bien passé, on copie les données dans les vraies variables
                self.Ymul1 = Ymul1
                self.Yoff1 = Yoff1
                self.voie1 = voie1
                self.voie1_old = voie1
                self.Ymul2 = Ymul2
                self.Yoff2 = Yoff2
                self.voie2 = voie2
                self.voie2_old = voie2
    
                self.infos.setText("Transfert terminé")
                self.infos.repaint()
                self.flag_ouvrir = False
                self.pasdesignal = False
                self.n = 2500
                self.curseur1.setMaximum(self.n-1)
                self.curseur2.setMaximum(self.n-1)
    
                self.mesures()
    
                self.pasdesignal = False
                self.plottedonc = True
                self.flag_ouvrir = False
                self.LED.setText("Oscillo OK")
                self.LED.setStyleSheet("""
                            .QLabel {
                                border: 2px solid maroon;
                                border-radius: 10px;
                                background-color: 'green';
                                color : 'yellow';
                                }
                            """)
                self.courbes_changees()
                self.plot()
            else:
                self.infos.setText("Aucune voie affichée à l'écran !")
                self.infos.repaint()
                self.message("Aucune voie affichée à l'écran !")
        except visa.VisaIOError:
            self.LED.setText("Pas d'oscillo")
            self.LED.setStyleSheet("""
                    .QLabel {
                        border: 2px solid maroon;
                        border-radius: 10px;
                        background-color: 'red';
                        color : 'yellow';
                        }
                    """)
            # self.buttonT.setEnabled(False)
            self.infos.setText("Si vous reconnectez l'oscilloscope, redémarrez le logiciel.")
            self.infos.repaint()
            self.message("Problème de connection à l'oscilloscope",
                         "Si vous reconnectez l'oscilloscope, redémarrez le logiciel.")

    def keyPressEvent(self, event):
        #  print('keypressEvent')
        #  réimplémente la méthode pour passer en manu en tapant entrée
        if type(event) == PyQt5.QtGui.QKeyEvent:
            if event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter:
                if self.SB_fmax.hasFocus():
                    if self.RB_fmax_manu.isChecked():
                        self.plot()
                    else:
                        self.RB_fmax_manu.setChecked(True)
                if self.SB_f.hasFocus():
                    if self.RB_aff.isChecked():
                        self.mesures()
                        self.plot()
                    else:
                        self.RB_aff.setChecked(True)

    def courbes_changees(self):
        #  rafraichit les autres pages si les signaux ont changé
        if self.spectres_ouvert: self.spectres()
        if self.filtrage_ouvert: self.fen_filtrage.calculer()
        if self.operations_ouvert: self.fen_operations.calculer()

    def plot(self):
        if self.plottedonc:
            #  print('plot')
            #  choisir les variables si affinée
            if self.flag_aff:
                v1 = self.voie1_aff
                v2 = self.voie2_aff
                Te = self.Te_aff
                if not self.RB_zoom.isChecked():
                    self.ntmax = 2 * int(1 / self.SB_f.value() / Te - 1)
            else:
                v1 = self.voie1
                v2 = self.voie2
                Te = self.Te
                if not self.RB_zoom.isChecked():
                    self.ntmax = self.n - 1
            Tt = arange(self.n) * Te

            # courbe nulle pour marquer l'axe
            nulle = linspace(0.0, 0.0, num=self.n)

            # discards the old graph
            self.axtempsCH1.clear()
            self.axtempsCH2.clear()

            # plot data
            if self.aff_ch1:
                self.axtempsCH1.plot(Tt[self.ntmin:self.ntmax], nulle[self.ntmin:self.ntmax], 'Maroon', linewidth=1)
                self.axtempsCH1.plot(Tt[self.ntmin:self.ntmax], v1[self.ntmin:self.ntmax], 'red', linewidth=1, label='CH1')
                #  curseur 1
                if self.voie_cur1.currentIndex() == 0:  #  sur CH1
                    t1 = Tt[self.curseur1.value()]
                    ord1 = v1[self.curseur1.value()]
                    self.axtempsCH1.plot(t1, ord1, self.couleur1, marker=">")
                    self.cur1_t.setText('t1 = ' + format(t1, '^9.2E') + 's   ')
                    self.cur1_v.setText('v1 = ' + format(ord1, '^9.2E') + 'V')

                #  curseur 2
                if self.voie_cur2.currentIndex() == 0:  #  sur CH1
                    t2 = Tt[self.curseur2.value()]
                    ord2 = v1[self.curseur2.value()]
                    self.axtempsCH1.plot(t2, ord2, self.couleur1, marker="<")
                    self.cur2_t.setText('t2 = ' + format(t2, '^9.2E') + 's   ')
                    self.cur2_v.setText('v2 = ' + format(ord2, '^9.2E') + 'V')

                self.axtempsCH1.set_ylabel("CH1 (V)", color='red')

            if self.aff_ch2:
                self.axtempsCH2.plot(Tt[self.ntmin:self.ntmax], nulle[self.ntmin:self.ntmax], 'Navy', linewidth=1)
                self.axtempsCH2.plot(Tt[self.ntmin:self.ntmax], v2[self.ntmin:self.ntmax], 'blue', linewidth=1, label='CH2')
                #  curseur 1
                if self.voie_cur1.currentIndex() == 1:  #  sur CH2
                    t1 = Tt[self.curseur1.value()]
                    ord1 = v2[self.curseur1.value()]
                    self.axtempsCH2.plot(t1, ord1, self.couleur2, marker=">")
                    self.cur1_t.setText('t1 = ' + format(t1, '^9.2E') + 's   ')
                    self.cur1_v.setText('v1 = ' + format(ord1, '^9.2E') + 'V')

                #  curseur 2
                if self.voie_cur2.currentIndex() == 1:  #  sur CH2
                    t2 = Tt[self.curseur2.value()]
                    ord2 = v2[self.curseur2.value()]
                    self.axtempsCH2.plot(t2, ord2, self.couleur2, marker="<")
                    self.cur2_t.setText('t2 = ' + format(t2, '^9.2E') + 's   ')
                    self.cur2_v.setText('v2 = ' + format(ord2, '^9.2E') + 'V')

                #  self.axtempsCH2.set_ylabel("CH2 (V)", color='blue')

                self.axtempsCH1.set_xlim(left=self.ntmin*Te, right=self.ntmax*Te)

            #  affichage des deltas et de l'intégrale

            t1 = Tt[self.curseur1.value()]  # au cas où une des courbes n'est pas affichée t1 ou t2 n'existe pas
            t2 = Tt[self.curseur2.value()]

            self.dx.setText('Δt = ' + format(t2 - t1, '^9.2E') + 's   ')
            if (self.aff_ch1 and self.voie_cur1.currentIndex() ==0 and self.voie_cur2.currentIndex() ==0) or \
                    (self.aff_ch2 and self.voie_cur1.currentIndex() ==1 and self.voie_cur2.currentIndex() ==1):
                self.dv.setText('Δv = ' + format(ord2 - ord1, '^9.2E') + 'V')
            else:
                self.dv.setText('Δv = ---V')

            if t2 != t1:
                self.f_mes.setText('f = ' + format(abs(1 / (t2 - t1)), '^9.2E') + 'Hz')
            else:
                self.f_mes.setText('f = --- Hz')

            if not self.flag_ouvrir:
                if self.aff_ch1:
                    self.axtempsCH1.set_ylim([(-127 - self.Yoff1) * self.Ymul1, (127 - self.Yoff1) * self.Ymul1])
                else:
                    self.axtempsCH1.set_ylim([(-127 - self.Yoff2) * self.Ymul2, (127 - self.Yoff2) * self.Ymul2])

                if self.aff_ch2:
                    self.axtempsCH2.set_ylim([(-127 - self.Yoff2) * self.Ymul2, (127 - self.Yoff2) * self.Ymul2])
                else:
                    self.axtempsCH2.set_ylim([(-127 - self.Yoff1) * self.Ymul1, (127 - self.Yoff1) * self.Ymul1])

            integrale = '---'

            if self.voie_cur1.currentIndex() == self.voie_cur2.currentIndex():
                if self.voie_cur1.currentIndex() == 0:

                    if self.curseur1.value() <= self.curseur2.value():
                        fonction = v1[self.curseur1.value():self.curseur2.value()]
                    else:
                        fonction = v1[self.curseur2.value():self.curseur1.value()]
                else:
                    if self.curseur1.value() <= self.curseur2.value():
                        fonction = v2[self.curseur1.value():self.curseur2.value()]
                    else:
                        fonction = v2[self.curseur2.value():self.curseur1.value()]
                integrale = format(Te * sum(fonction), '9.2E')
            self.label_integrale.setText('Intégrale entre les curseurs : ' + integrale + ' Vs')

            phi = '---'

            if (self.voie_cur2.currentIndex() == 0) or (self.voie_cur1.currentIndex() == 0):
                phi = format((abs(t2 - t1) * self.f1 * 360) % 360, '4.1f')
            else:
                phi = format((abs(t2 - t1) * self.f2 * 360) % 360, '4.1f')
            self.label_phi.setText('φ = ' + phi + ' °')

            # légendes
            chartBoxtemps = self.axtempsCH1.get_position()

            #  self.axtempsCH1.set_position(
            #    [chartBoxtemps.x0, chartBoxtemps.y0, chartBoxFFT.width * 0.75, chartBoxFFT.height * 0.9])
            #  self.axtempsCH2.set_position(
            #    [chartBoxtemps.x0, chartBoxtemps.y0, chartBoxFFT.width * 0.75, chartBoxFFT.height * 0.9])
            #  self.axtempsCH1.legend(loc='upper center', bbox_to_anchor=(1.35, 0.8), shadow=True, ncol=1)
            #  self.axtempsCH2.legend(loc='upper center', bbox_to_anchor=(1.35, 0.4), shadow=True, ncol=1)
            self.axtempsCH1.set_xlabel("temps (s)")
            self.axtempsCH1.grid(True)

            # refresh canvas
            self.canvas1.draw()

    def trouveVisa(self):
        self.rm.open_resource  #  essayer de mettre ressource = ... puis à la fin de fermer
        a = self.rm.list_resources()
        print('ressources visa : ', a)
        self.adresseVisa = 'vide'
        reponse = ''
        for chaine in a:
            try:
                inst = self.rm.open_resource(chaine)
                reponse = inst.query("*IDN?")
            except visa.VisaIOError:
                pass
            else:
                if reponse.find('TEKTRONIX') > -1:
                    self.adresseVisa = chaine
                    if reponse.find('TBS') > -1: self.type_oscillo = 'TBS'
                    else: self.type_oscillo = 'TDS'
                print(self.type_oscillo)
        print('Adresse visa oscillo = ', self.adresseVisa)
        if self.adresseVisa == 'vide':
            self.LED.setText("Pas d'oscillo")
            self.LED.setStyleSheet("""
            .QLabel {
                border: 2px solid maroon;
                border-radius: 10px;
                background-color: 'red';
                color : 'yellow';
                }
            """)
            # self.buttonT.setEnabled(False)
        else:
            self.LED.setText("Oscillo OK")
            self.LED.setStyleSheet("""
            .QLabel {
                border: 2px solid maroon;
                border-radius: 10px;
                background-color: 'green';
                color : 'yellow';
                }
            """)

    def closeEvent(self, QCloseEvent):
        if self.spectres_ouvert:
            self.page_fft.close()
        if self.filtrage_ouvert:
            self.fen_filtrage.close()
        if self.operations_ouvert:
            self.fen_operations.close()


if __name__ == '__main__':
    print('Lancement du logiciel...')
    app = QApplication(sys.argv)

    main = Principale()
    main.show()
    app.exec_()
